@@ -321,7 +321,7 @@ nsresult NSSSocketControl::ActivateSSL() {
 
   mHandshakePending = true;
 
-  return SetResumptionTokenFromExternalCache();
+  return SetResumptionTokenFromExternalCache(mFd);
 }
 
 nsresult NSSSocketControl::GetFileDescPtr(PRFileDesc** aFilePtr) {
@@ -382,7 +382,15 @@ void NSSSocketControl::SetCertVerificationResult(PRErrorCode errorCode) {
                           AssertedCast<uint32_t>(mPlaintextBytesRead));
   }
 
+  MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
+          ("[%p] SetCertVerificationResult to AfterCertVerification, "
+           "mTlsHandshakeCallback=%p",
+           (void*)mFd, mTlsHandshakeCallback.get()));
+
   mCertVerificationState = AfterCertVerification;
+  if (mTlsHandshakeCallback) {
+    Unused << mTlsHandshakeCallback->CertVerificationDone();
+  }
 }
 
 void NSSSocketControl::ClientAuthCertificateSelected(
@@ -434,6 +442,13 @@ void NSSSocketControl::ClientAuthCertificateSelected(
       mFd, sendingClientAuthCert ? SECSuccess : SECFailure,
       sendingClientAuthCert ? key.release() : nullptr,
       sendingClientAuthCert ? cert.release() : nullptr);
+
+  MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
+          ("[%p] ClientAuthCertificateSelected mTlsHandshakeCallback=%p",
+           (void*)mFd, mTlsHandshakeCallback.get()));
+  if (mTlsHandshakeCallback) {
+    Unused << mTlsHandshakeCallback->ClientAuthCertificateSelected();
+  }
 }
 
 SharedSSLState& NSSSocketControl::SharedState() {
@@ -508,7 +523,6 @@ PRStatus NSSSocketControl::CloseSocketAndDestroy() {
   if (status != PR_SUCCESS) return status;
 
   popped->identity = PR_INVALID_IO_LAYER;
-  NS_RELEASE_THIS();
   popped->dtor(popped);
 
   return PR_SUCCESS;
@@ -624,15 +638,15 @@ NSSSocketControl::GetPeerId(nsACString& aResult) {
   return NS_OK;
 }
 
-nsresult NSSSocketControl::SetResumptionTokenFromExternalCache() {
+nsresult NSSSocketControl::SetResumptionTokenFromExternalCache(PRFileDesc* fd) {
   COMMON_SOCKET_CONTROL_ASSERT_ON_OWNING_THREAD();
-  if (!mFd) {
-    return NS_ERROR_FAILURE;
+  if (!fd) {
+    return NS_ERROR_INVALID_ARG;
   }
 
   // If SSL_NO_CACHE option was set, we must not use the cache
   PRIntn val;
-  if (SSL_OptionGet(mFd, SSL_NO_CACHE, &val) != SECSuccess) {
+  if (SSL_OptionGet(fd, SSL_NO_CACHE, &val) != SECSuccess) {
     return NS_ERROR_FAILURE;
   }
 
@@ -659,7 +673,7 @@ nsresult NSSSocketControl::SetResumptionTokenFromExternalCache() {
     return rv;
   }
 
-  SECStatus srv = SSL_SetResumptionToken(mFd, token.Elements(), token.Length());
+  SECStatus srv = SSL_SetResumptionToken(fd, token.Elements(), token.Length());
   if (srv == SECFailure) {
     PRErrorCode error = PR_GetError();
     mozilla::net::SSLTokensCache::Remove(peerId, tokenId);
