@@ -65,11 +65,10 @@
 //
 // SSLServerCertVerificationResult must be dispatched to the socket transport
 // thread because we must only call SSL_* functions on the socket transport
-// thread since they may do I/O, because many parts of nsNSSSocketInfo (the
-// subclass of TransportSecurityInfo used when validating certificates during
-// an SSL handshake) and the PSM NSS I/O layer are not thread-safe, and because
-// we need the event to interrupt the PR_Poll that may waiting for I/O on the
-// socket for which we are validating the cert.
+// thread since they may do I/O, because many parts of NSSSocketControl and the
+// PSM NSS I/O layer are not thread-safe, and because we need the event to
+// interrupt the PR_Poll that may waiting for I/O on the socket for which we
+// are validating the cert.
 //
 // When socket process is enabled, libssl is running on socket process. To
 // perform certificate authentication with CertVerifier, we have to send all
@@ -99,12 +98,12 @@
 #include "CryptoTask.h"
 #include "ExtendedValidation.h"
 #include "NSSCertDBTrustDomain.h"
+#include "NSSSocketControl.h"
 #include "PSMRunnable.h"
 #include "RootCertificateTelemetryUtils.h"
 #include "ScopedNSSTypes.h"
 #include "SharedCertVerifier.h"
 #include "SharedSSLState.h"
-#include "TransportSecurityInfo.h"  // For RememberCertErrorsTable
 #include "VerifySSLServerCertChild.h"
 #include "cert.h"
 #include "mozilla/Assertions.h"
@@ -131,6 +130,7 @@
 #include "mozpkix/pkix.h"
 #include "mozpkix/pkixcheck.h"
 #include "mozpkix/pkixnss.h"
+#include "mozpkix/pkixutil.h"
 #include "secerr.h"
 #include "secport.h"
 #include "ssl.h"
@@ -144,13 +144,9 @@ using namespace mozilla::pkix;
 namespace mozilla {
 namespace psm {
 
-namespace {
-
 // do not use a nsCOMPtr to avoid static initializer/destructor
 nsIThreadPool* gCertVerificationThreadPool = nullptr;
 
-}  // unnamed namespace
-
 // Called when the socket transport thread starts, to initialize the SSL cert
 // verification thread pool. By tying the thread pool startup/shutdown directly
 // to the STS thread's lifetime, we ensure that they are *always* available for
@@ -189,8 +185,6 @@ void StopSSLServerCertVerificationThreads() {
   }
 }
 
-namespace {
-
 // A probe value of 1 means "no error".
 uint32_t MapOverridableErrorToProbeValue(PRErrorCode errorCode) {
   switch (errorCode) {
@@ -235,7 +229,7 @@ uint32_t MapOverridableErrorToProbeValue(PRErrorCode errorCode) {
   }
   NS_WARNING(
       "Unknown certificate error code. Does MapOverridableErrorToProbeValue "
-      "handle everything in DetermineCertOverrideErrors?");
+      "handle everything in CategorizeCertificateError?");
   return 0;
 }
 
@@ -269,35 +263,12 @@ static uint32_t MapCertErrorToProbeValue(PRErrorCode errorCode) {
   return probeValue;
 }
 
-SECStatus DetermineCertOverrideErrors(const nsCOMPtr<nsIX509Cert>& cert,
-                                      const nsACString& hostName,
-                                      mozilla::pkix::Time now,
-                                      PRErrorCode defaultErrorCodeToReport,
-                                      /*out*/ uint32_t& collectedErrors,
-                                      /*out*/ PRErrorCode& errorCodeTrust,
-                                      /*out*/ PRErrorCode& errorCodeMismatch,
-                                      /*out*/ PRErrorCode& errorCodeTime) {
-  MOZ_ASSERT(cert);
-  MOZ_ASSERT(collectedErrors == 0);
-  MOZ_ASSERT(errorCodeTrust == 0);
-  MOZ_ASSERT(errorCodeMismatch == 0);
-  MOZ_ASSERT(errorCodeTime == 0);
-
-  nsTArray<uint8_t> certDER;
-  if (NS_FAILED(cert->GetRawDER(certDER))) {
-    PR_SetError(SEC_ERROR_LIBRARY_FAILURE, 0);
-    return SECFailure;
-  }
-  mozilla::pkix::Input certInput;
-  if (certInput.Init(certDER.Elements(), certDER.Length()) != Success) {
-    PR_SetError(SEC_ERROR_BAD_DER, 0);
-    return SECFailure;
-  }
-
-  // Assumes the error prioritization described in mozilla::pkix's
-  // BuildForward function. Also assumes that CheckCertHostname was only
-  // called if CertVerifier::VerifyCert succeeded.
-  switch (defaultErrorCodeToReport) {
+// If the given PRErrorCode is an overridable certificate error, return which
+// category (trust, time, domain mismatch) it falls in. If it is not
+// overridable, return Nothing.
+Maybe<nsITransportSecurityInfo::OverridableErrorCategory>
+CategorizeCertificateError(PRErrorCode certificateError) {
+  switch (certificateError) {
     case SEC_ERROR_CERT_SIGNATURE_ALGORITHM_DISABLED:
     case SEC_ERROR_EXPIRED_ISSUER_CERTIFICATE:
     case SEC_ERROR_UNKNOWN_ISSUER:
@@ -309,95 +280,24 @@ SECStatus DetermineCertOverrideErrors(const nsCOMPtr<nsIX509Cert>& cert,
     case mozilla::pkix::MOZILLA_PKIX_ERROR_MITM_DETECTED:
     case mozilla::pkix::MOZILLA_PKIX_ERROR_NOT_YET_VALID_ISSUER_CERTIFICATE:
     case mozilla::pkix::MOZILLA_PKIX_ERROR_SELF_SIGNED_CERT:
-    case mozilla::pkix::MOZILLA_PKIX_ERROR_V1_CERT_USED_AS_CA: {
-      collectedErrors = nsICertOverrideService::ERROR_UNTRUSTED;
-      errorCodeTrust = defaultErrorCodeToReport;
+    case mozilla::pkix::MOZILLA_PKIX_ERROR_V1_CERT_USED_AS_CA:
+      return Some(
+          nsITransportSecurityInfo::OverridableErrorCategory::ERROR_TRUST);
 
-      mozilla::pkix::BackCert backCert(
-          certInput, mozilla::pkix::EndEntityOrCA::MustBeEndEntity, nullptr);
-      Result rv = backCert.Init();
-      if (rv != Success) {
-        PR_SetError(MapResultToPRErrorCode(rv), 0);
-        return SECFailure;
-      }
-      mozilla::pkix::Time notBefore(mozilla::pkix::Time::uninitialized);
-      mozilla::pkix::Time notAfter(mozilla::pkix::Time::uninitialized);
-      // If the validity can't be parsed, ParseValidity will return
-      // Result::ERROR_INVALID_DER_TIME.
-      rv = mozilla::pkix::ParseValidity(backCert.GetValidity(), &notBefore,
-                                        &notAfter);
-      if (rv != Success) {
-        collectedErrors |= nsICertOverrideService::ERROR_TIME;
-        errorCodeTime = MapResultToPRErrorCode(rv);
-        break;
-      }
-      // If `now` is outside of the certificate's validity period,
-      // CheckValidity will return Result::ERROR_NOT_YET_VALID_CERTIFICATE or
-      // Result::ERROR_EXPIRED_CERTIFICATE, as appropriate, and Success
-      // otherwise.
-      rv = mozilla::pkix::CheckValidity(now, notBefore, notAfter);
-      if (rv != Success) {
-        collectedErrors |= nsICertOverrideService::ERROR_TIME;
-        errorCodeTime = MapResultToPRErrorCode(rv);
-      }
-      break;
-    }
+    case SSL_ERROR_BAD_CERT_DOMAIN:
+      return Some(
+          nsITransportSecurityInfo::OverridableErrorCategory::ERROR_DOMAIN);
 
     case SEC_ERROR_INVALID_TIME:
     case SEC_ERROR_EXPIRED_CERTIFICATE:
     case mozilla::pkix::MOZILLA_PKIX_ERROR_NOT_YET_VALID_CERTIFICATE:
-      collectedErrors = nsICertOverrideService::ERROR_TIME;
-      errorCodeTime = defaultErrorCodeToReport;
-      break;
-
-    case SSL_ERROR_BAD_CERT_DOMAIN:
-      collectedErrors = nsICertOverrideService::ERROR_MISMATCH;
-      errorCodeMismatch = SSL_ERROR_BAD_CERT_DOMAIN;
-      break;
-
-    case 0:
-      NS_ERROR("No error code set during certificate validation failure.");
-      PR_SetError(PR_INVALID_STATE_ERROR, 0);
-      return SECFailure;
+      return Some(
+          nsITransportSecurityInfo::OverridableErrorCategory::ERROR_TIME);
 
     default:
-      PR_SetError(defaultErrorCodeToReport, 0);
-      return SECFailure;
+      break;
   }
-
-  if (defaultErrorCodeToReport != SSL_ERROR_BAD_CERT_DOMAIN) {
-    Input hostnameInput;
-    Result result = hostnameInput.Init(
-        BitwiseCast<const uint8_t*, const char*>(hostName.BeginReading()),
-        hostName.Length());
-    if (result != Success) {
-      PR_SetError(SEC_ERROR_INVALID_ARGS, 0);
-      return SECFailure;
-    }
-    // CheckCertHostname expects that its input represents a certificate that
-    // has already been successfully validated by BuildCertChain. This is
-    // obviously not the case, however, because we're in the error path of
-    // certificate verification. Thus, this is problematic. In the future, it
-    // would be nice to remove this optimistic additional error checking and
-    // simply punt to the front-end, which can more easily (and safely) perform
-    // extra checks to give the user hints as to why verification failed.
-    result = CheckCertHostname(certInput, hostnameInput);
-    // Treat malformed name information as a domain mismatch.
-    if (result == Result::ERROR_BAD_DER ||
-        result == Result::ERROR_BAD_CERT_DOMAIN) {
-      collectedErrors |= nsICertOverrideService::ERROR_MISMATCH;
-      errorCodeMismatch = SSL_ERROR_BAD_CERT_DOMAIN;
-    } else if (IsFatalError(result)) {
-      // Because its input has not been validated by BuildCertChain,
-      // CheckCertHostname can return an error that is less important than the
-      // original certificate verification error. Only return an error result
-      // from this function if we've encountered a fatal error.
-      PR_SetError(MapResultToPRErrorCode(result), 0);
-      return SECFailure;
-    }
-  }
-
-  return SECSuccess;
+  return Nothing();
 }
 
 // Helper function to determine if overrides are allowed for this host.
@@ -441,8 +341,8 @@ static nsresult OverrideAllowedForHost(
     return rv;
   }
 
-  rv = sss->IsSecureURI(uri, aOriginAttributes, nullptr, nullptr,
-                        &strictTransportSecurityEnabled);
+  rv =
+      sss->IsSecureURI(uri, aOriginAttributes, &strictTransportSecurityEnabled);
   if (NS_FAILED(rv)) {
     MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
             ("[0x%" PRIx64 "] checking for HSTS failed", aPtrForLog));
@@ -481,16 +381,23 @@ static nsresult OverrideAllowedForHost(
 // Prohibit changing the server cert only if we negotiated SPDY,
 // in order to support SPDY's cross-origin connection pooling.
 static SECStatus BlockServerCertChangeForSpdy(
-    nsNSSSocketInfo* infoObject, const UniqueCERTCertificate& serverCert) {
-  if (!infoObject->IsHandshakeCompleted()) {
+    NSSSocketControl* socketControl, const UniqueCERTCertificate& serverCert) {
+  if (!socketControl->IsHandshakeCompleted()) {
     // first handshake on this connection, not a
     // renegotiation.
     return SECSuccess;
   }
 
   // Filter out sockets that did not neogtiate SPDY via NPN
+  nsCOMPtr<nsITransportSecurityInfo> securityInfo;
+  nsresult rv = socketControl->GetSecurityInfo(getter_AddRefs(securityInfo));
+  MOZ_ASSERT(NS_SUCCEEDED(rv), "GetSecurityInfo() failed during renegotiation");
+  if (NS_FAILED(rv) || !securityInfo) {
+    PR_SetError(SEC_ERROR_LIBRARY_FAILURE, 0);
+    return SECFailure;
+  }
   nsAutoCString negotiatedNPN;
-  nsresult rv = infoObject->GetNegotiatedNPN(negotiatedNPN);
+  rv = securityInfo->GetNegotiatedNPN(negotiatedNPN);
   MOZ_ASSERT(NS_SUCCEEDED(rv),
              "GetNegotiatedNPN() failed during renegotiation");
 
@@ -505,8 +412,7 @@ static SECStatus BlockServerCertChangeForSpdy(
   }
 
   // Check to see if the cert has actually changed
-  nsCOMPtr<nsIX509Cert> cert;
-  infoObject->GetServerCert(getter_AddRefs(cert));
+  nsCOMPtr<nsIX509Cert> cert(socketControl->GetServerCert());
   if (!cert) {
     PR_SetError(SEC_ERROR_LIBRARY_FAILURE, 0);
     return SECFailure;
@@ -641,7 +547,7 @@ void GatherCertificateTransparencyTelemetry(
 static void CollectCertTelemetry(
     mozilla::pkix::Result aCertVerificationResult, EVStatus aEVStatus,
     CertVerifier::OCSPStaplingStatus aOcspStaplingStatus,
-    KeySizeStatus aKeySizeStatus, SHA1ModeResult aSha1ModeResult,
+    KeySizeStatus aKeySizeStatus,
     const PinningTelemetryInfo& aPinningTelemetryInfo,
     const nsTArray<nsTArray<uint8_t>>& aBuiltCertChain,
     const CertificateTransparencyInfo& aCertificateTransparencyInfo) {
@@ -659,11 +565,6 @@ static void CollectCertTelemetry(
                           static_cast<uint32_t>(aKeySizeStatus));
   }
 
-  if (aSha1ModeResult != SHA1ModeResult::NeverChecked) {
-    Telemetry::Accumulate(Telemetry::CERT_CHAIN_SHA1_POLICY_STATUS,
-                          static_cast<uint32_t>(aSha1ModeResult));
-  }
-
   if (aPinningTelemetryInfo.accumulateForRoot) {
     Telemetry::Accumulate(Telemetry::CERT_PINNING_FAILURES_BY_CA,
                           aPinningTelemetryInfo.rootBucket);
@@ -698,11 +599,11 @@ Result AuthCertificate(
     /*out*/ nsTArray<nsTArray<uint8_t>>& builtCertChain,
     /*out*/ EVStatus& evStatus,
     /*out*/ CertificateTransparencyInfo& certificateTransparencyInfo,
-    /*out*/ bool& aIsBuiltCertChainRootBuiltInRoot) {
+    /*out*/ bool& aIsBuiltCertChainRootBuiltInRoot,
+    /*out*/ bool& aMadeOCSPRequests) {
   CertVerifier::OCSPStaplingStatus ocspStaplingStatus =
       CertVerifier::OCSP_STAPLING_NEVER_CHECKED;
   KeySizeStatus keySizeStatus = KeySizeStatus::NeverChecked;
-  SHA1ModeResult sha1ModeResult = SHA1ModeResult::NeverChecked;
   PinningTelemetryInfo pinningTelemetryInfo;
 
   nsTArray<nsTArray<uint8_t>> peerCertsBytes;
@@ -718,12 +619,12 @@ Result AuthCertificate(
       certBytes, time, aPinArg, aHostName, builtCertChain, certVerifierFlags,
       Some(std::move(peerCertsBytes)), stapledOCSPResponse,
       sctsFromTLSExtension, dcInfo, aOriginAttributes, &evStatus,
-      &ocspStaplingStatus, &keySizeStatus, &sha1ModeResult,
-      &pinningTelemetryInfo, &certificateTransparencyInfo,
-      &aIsBuiltCertChainRootBuiltInRoot);
+      &ocspStaplingStatus, &keySizeStatus, &pinningTelemetryInfo,
+      &certificateTransparencyInfo, &aIsBuiltCertChainRootBuiltInRoot,
+      &aMadeOCSPRequests);
 
   CollectCertTelemetry(rv, evStatus, ocspStaplingStatus, keySizeStatus,
-                       sha1ModeResult, pinningTelemetryInfo, builtCertChain,
+                       pinningTelemetryInfo, builtCertChain,
                        certificateTransparencyInfo);
 
   return rv;
@@ -733,112 +634,63 @@ PRErrorCode AuthCertificateParseResults(
     uint64_t aPtrForLog, const nsACString& aHostName, int32_t aPort,
     const OriginAttributes& aOriginAttributes,
     const nsCOMPtr<nsIX509Cert>& aCert, mozilla::pkix::Time aTime,
-    PRErrorCode aDefaultErrorCodeToReport,
-    /* out */ uint32_t& aCollectedErrors) {
-  if (aDefaultErrorCodeToReport == 0) {
-    MOZ_ASSERT_UNREACHABLE(
-        "No error set during certificate validation failure");
-    return SEC_ERROR_LIBRARY_FAILURE;
-  }
-
-  uint32_t probeValue = MapCertErrorToProbeValue(aDefaultErrorCodeToReport);
+    PRErrorCode aCertVerificationError,
+    /* out */
+    nsITransportSecurityInfo::OverridableErrorCategory&
+        aOverridableErrorCategory) {
+  uint32_t probeValue = MapCertErrorToProbeValue(aCertVerificationError);
   Telemetry::Accumulate(Telemetry::SSL_CERT_VERIFICATION_ERRORS, probeValue);
 
-  aCollectedErrors = 0;
-  PRErrorCode errorCodeTrust = 0;
-  PRErrorCode errorCodeMismatch = 0;
-  PRErrorCode errorCodeTime = 0;
-  if (DetermineCertOverrideErrors(
-          aCert, aHostName, aTime, aDefaultErrorCodeToReport, aCollectedErrors,
-          errorCodeTrust, errorCodeMismatch, errorCodeTime) != SECSuccess) {
-    PRErrorCode errorCode = PR_GetError();
-    MOZ_ASSERT(!ErrorIsOverridable(errorCode));
-    if (errorCode == 0) {
-      MOZ_ASSERT_UNREACHABLE(
-          "No error set during DetermineCertOverrideErrors failure");
-      return SEC_ERROR_LIBRARY_FAILURE;
-    }
-    return errorCode;
-  }
-
-  if (!aCollectedErrors) {
-    MOZ_ASSERT_UNREACHABLE("aCollectedErrors should not be 0");
-    return SEC_ERROR_LIBRARY_FAILURE;
+  Maybe<nsITransportSecurityInfo::OverridableErrorCategory>
+      maybeOverridableErrorCategory =
+          CategorizeCertificateError(aCertVerificationError);
+  // If this isn't an overridable error, return it now. This will stop the
+  // connection and report the given error.
+  if (!maybeOverridableErrorCategory.isSome()) {
+    return aCertVerificationError;
   }
+  aOverridableErrorCategory = *maybeOverridableErrorCategory;
 
   bool overrideAllowed = false;
-  if (NS_FAILED(OverrideAllowedForHost(aPtrForLog, aHostName, aOriginAttributes,
-                                       overrideAllowed))) {
-    MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
-            ("[0x%" PRIx64 "] AuthCertificateParseResults - "
-             "OverrideAllowedForHost failed\n",
-             aPtrForLog));
-    return aDefaultErrorCodeToReport;
+  nsresult rv = OverrideAllowedForHost(aPtrForLog, aHostName, aOriginAttributes,
+                                       overrideAllowed);
+  if (NS_FAILED(rv)) {
+    return aCertVerificationError;
   }
 
-  if (overrideAllowed) {
-    nsCOMPtr<nsICertOverrideService> overrideService =
-        do_GetService(NS_CERTOVERRIDE_CONTRACTID);
-
-    uint32_t overrideBits = 0;
-    uint32_t remainingDisplayErrors = aCollectedErrors;
-
-    // it is fine to continue without the nsICertOverrideService
-    if (overrideService) {
-      bool haveOverride;
-      bool isTemporaryOverride;  // we don't care
-      nsresult rv = overrideService->HasMatchingOverride(
-          aHostName, aPort, aOriginAttributes, aCert, &overrideBits,
-          &isTemporaryOverride, &haveOverride);
-      if (NS_SUCCEEDED(rv) && haveOverride) {
-        // remove the errors that are already overriden
-        remainingDisplayErrors &= ~overrideBits;
-      }
-    }
-
-    if (!remainingDisplayErrors) {
-      // This can double- or triple-count one certificate with multiple
-      // different types of errors. Since this is telemetry and we just
-      // want a ballpark answer, we don't care.
-      if (errorCodeTrust != 0) {
-        uint32_t probeValue = MapOverridableErrorToProbeValue(errorCodeTrust);
-        Telemetry::Accumulate(Telemetry::SSL_CERT_ERROR_OVERRIDES, probeValue);
-      }
-      if (errorCodeMismatch != 0) {
-        uint32_t probeValue =
-            MapOverridableErrorToProbeValue(errorCodeMismatch);
-        Telemetry::Accumulate(Telemetry::SSL_CERT_ERROR_OVERRIDES, probeValue);
-      }
-      if (errorCodeTime != 0) {
-        uint32_t probeValue = MapOverridableErrorToProbeValue(errorCodeTime);
-        Telemetry::Accumulate(Telemetry::SSL_CERT_ERROR_OVERRIDES, probeValue);
-      }
-
-      // all errors are covered by override rules, so let's accept the cert
-      MOZ_LOG(
-          gPIPNSSLog, LogLevel::Debug,
-          ("[0x%" PRIx64 "] All errors covered by override rules", aPtrForLog));
-      return 0;
-    }
-  } else {
+  if (!overrideAllowed) {
     MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
-            ("[0x%" PRIx64 "] HSTS or pinned host - no overrides allowed\n",
+            ("[0x%" PRIx64 "] HSTS or pinned host - no overrides allowed",
              aPtrForLog));
+    return aCertVerificationError;
   }
 
-  MOZ_LOG(
-      gPIPNSSLog, LogLevel::Debug,
-      ("[0x%" PRIx64 "] Certificate error was not overridden\n", aPtrForLog));
+  nsCOMPtr<nsICertOverrideService> overrideService =
+      do_GetService(NS_CERTOVERRIDE_CONTRACTID);
+  if (!overrideService) {
+    return aCertVerificationError;
+  }
+  bool haveOverride;
+  bool isTemporaryOverride;
+  rv = overrideService->HasMatchingOverride(aHostName, aPort, aOriginAttributes,
+                                            aCert, &isTemporaryOverride,
+                                            &haveOverride);
+  if (NS_FAILED(rv)) {
+    return aCertVerificationError;
+  }
+  Unused << isTemporaryOverride;
+  if (haveOverride) {
+    uint32_t probeValue =
+        MapOverridableErrorToProbeValue(aCertVerificationError);
+    Telemetry::Accumulate(Telemetry::SSL_CERT_ERROR_OVERRIDES, probeValue);
+    MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
+            ("[0x%" PRIx64 "] certificate error overridden", aPtrForLog));
+    return 0;
+  }
 
-  // pick the error code to report by priority
-  return errorCodeTrust      ? errorCodeTrust
-         : errorCodeMismatch ? errorCodeMismatch
-         : errorCodeTime     ? errorCodeTime
-                             : aDefaultErrorCodeToReport;
+  return aCertVerificationError;
 }
 
-}  // unnamed namespace
-
 static nsTArray<nsTArray<uint8_t>> CreateCertBytesArray(
     const UniqueCERTCertList& aCertChain) {
   nsTArray<nsTArray<uint8_t>> certsBytes;
@@ -916,13 +768,15 @@ SSLServerCertVerificationJob::Run() {
   EVStatus evStatus;
   CertificateTransparencyInfo certificateTransparencyInfo;
   bool isCertChainRootBuiltInRoot = false;
+  bool madeOCSPRequests = false;
   nsTArray<nsTArray<uint8_t>> builtChainBytesArray;
   nsTArray<uint8_t> certBytes(mPeerCertChain.ElementAt(0).Clone());
   Result rv = AuthCertificate(
       *certVerifier, mPinArg, certBytes, mPeerCertChain, mHostName,
       mOriginAttributes, mStapledOCSPResponse, mSCTsFromTLSExtension, mDCInfo,
       mProviderFlags, mTime, mCertVerifierFlags, builtChainBytesArray, evStatus,
-      certificateTransparencyInfo, isCertChainRootBuiltInRoot);
+      certificateTransparencyInfo, isCertChainRootBuiltInRoot,
+      madeOCSPRequests);
 
   if (rv == Success) {
     Telemetry::AccumulateTimeDelta(
@@ -934,7 +788,9 @@ SSLServerCertVerificationJob::Run() {
         std::move(builtChainBytesArray), std::move(mPeerCertChain),
         TransportSecurityInfo::ConvertCertificateTransparencyInfoToStatus(
             certificateTransparencyInfo),
-        evStatus, true, 0, 0, isCertChainRootBuiltInRoot, mProviderFlags);
+        evStatus, true, 0,
+        nsITransportSecurityInfo::OverridableErrorCategory::ERROR_UNSET,
+        isCertChainRootBuiltInRoot, mProviderFlags, madeOCSPRequests);
     return NS_OK;
   }
 
@@ -943,25 +799,26 @@ SSLServerCertVerificationJob::Run() {
       jobStartTime, TimeStamp::Now());
 
   PRErrorCode error = MapResultToPRErrorCode(rv);
-  uint32_t collectedErrors = 0;
+  nsITransportSecurityInfo::OverridableErrorCategory overridableErrorCategory =
+      nsITransportSecurityInfo::OverridableErrorCategory::ERROR_UNSET;
   nsCOMPtr<nsIX509Cert> cert(new nsNSSCertificate(std::move(certBytes)));
   PRErrorCode finalError = AuthCertificateParseResults(
       mAddrForLogging, mHostName, mPort, mOriginAttributes, cert, mTime, error,
-      collectedErrors);
+      overridableErrorCategory);
 
   // NB: finalError may be 0 here, in which the connection will continue.
   mResultTask->Dispatch(
       std::move(builtChainBytesArray), std::move(mPeerCertChain),
       nsITransportSecurityInfo::CERTIFICATE_TRANSPARENCY_NOT_APPLICABLE,
-      EVStatus::NotEV, false, finalError, collectedErrors, false,
-      mProviderFlags);
+      EVStatus::NotEV, false, finalError, overridableErrorCategory, false,
+      mProviderFlags, madeOCSPRequests);
   return NS_OK;
 }
 
 // Takes information needed for cert verification, does some consistency
 //  checks and calls SSLServerCertVerificationJob::Dispatch.
 SECStatus AuthCertificateHookInternal(
-    TransportSecurityInfo* infoObject, const void* aPtrForLogging,
+    CommonSocketControl* socketControl, const void* aPtrForLogging,
     const nsACString& hostName, nsTArray<nsTArray<uint8_t>>&& peerCertChain,
     Maybe<nsTArray<uint8_t>>& stapledOCSPResponse,
     Maybe<nsTArray<uint8_t>>& sctsFromTLSExtension,
@@ -972,7 +829,7 @@ SECStatus AuthCertificateHookInternal(
   MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
           ("[%p] starting AuthCertificateHookInternal\n", aPtrForLogging));
 
-  if (!infoObject || peerCertChain.IsEmpty()) {
+  if (!socketControl || peerCertChain.IsEmpty()) {
     PR_SetError(PR_INVALID_STATE_ERROR, 0);
     return SECFailure;
   }
@@ -1000,12 +857,12 @@ SECStatus AuthCertificateHookInternal(
 
   uint64_t addr = reinterpret_cast<uintptr_t>(aPtrForLogging);
   RefPtr<SSLServerCertVerificationResult> resultTask =
-      new SSLServerCertVerificationResult(infoObject);
+      new SSLServerCertVerificationResult(socketControl);
 
   if (XRE_IsSocketProcess()) {
     return RemoteProcessCertVerification(
-        std::move(peerCertChain), hostName, infoObject->GetPort(),
-        infoObject->GetOriginAttributes(), stapledOCSPResponse,
+        std::move(peerCertChain), hostName, socketControl->GetPort(),
+        socketControl->GetOriginAttributes(), stapledOCSPResponse,
         sctsFromTLSExtension, dcInfo, providerFlags, certVerifierFlags,
         resultTask);
   }
@@ -1015,8 +872,8 @@ SECStatus AuthCertificateHookInternal(
   // and we *want* to do certificate verification on a background thread
   // because of the performance benefits of doing so.
   return SSLServerCertVerificationJob::Dispatch(
-      addr, infoObject, std::move(peerCertChain), hostName,
-      infoObject->GetPort(), infoObject->GetOriginAttributes(),
+      addr, socketControl, std::move(peerCertChain), hostName,
+      socketControl->GetPort(), socketControl->GetOriginAttributes(),
       stapledOCSPResponse, sctsFromTLSExtension, dcInfo, providerFlags, Now(),
       certVerifierFlags, resultTask);
 }
@@ -1038,7 +895,7 @@ SECStatus AuthCertificateHook(void* arg, PRFileDesc* fd, PRBool checkSig,
   // and many things in PSM assume that we are a client.
   MOZ_ASSERT(!isServer, "AuthCertificateHook: isServer unexpectedly true");
 
-  nsNSSSocketInfo* socketInfo = static_cast<nsNSSSocketInfo*>(arg);
+  NSSSocketControl* socketInfo = static_cast<NSSSocketControl*>(arg);
 
   UniqueCERTCertificate serverCert(SSL_PeerCertificate(fd));
 
@@ -1130,7 +987,7 @@ SECStatus AuthCertificateHook(void* arg, PRFileDesc* fd, PRBool checkSig,
 // checks and calls SSLServerCertVerificationJob::Dispatch.
 // This function is used for Quic.
 SECStatus AuthCertificateHookWithInfo(
-    TransportSecurityInfo* infoObject, const nsACString& aHostName,
+    CommonSocketControl* socketControl, const nsACString& aHostName,
     const void* aPtrForLogging, nsTArray<nsTArray<uint8_t>>&& peerCertChain,
     Maybe<nsTArray<nsTArray<uint8_t>>>& stapledOCSPResponses,
     Maybe<nsTArray<uint8_t>>& sctsFromTLSExtension, uint32_t providerFlags) {
@@ -1146,7 +1003,7 @@ SECStatus AuthCertificateHookWithInfo(
   }
 
   uint32_t certVerifierFlags = 0;
-  // QuicTransportSecInfo does not have a SharedState as nsNSSSocketInfo.
+  // QuicSocketControl does not have a SharedState as NSSSocketControl.
   // Here we need prefs for ocsp. This are prefs they are the same for
   // PublicSSLState and PrivateSSLState, just take them from one of them.
   if (!PublicSSLState()->IsOCSPStaplingEnabled() ||
@@ -1158,7 +1015,7 @@ SECStatus AuthCertificateHookWithInfo(
   // for Delegated Credentials.
   Maybe<DelegatedCredentialInfo> dcInfo;
 
-  return AuthCertificateHookInternal(infoObject, aPtrForLogging, aHostName,
+  return AuthCertificateHookInternal(socketControl, aPtrForLogging, aHostName,
                                      std::move(peerCertChain),
                                      stapledOCSPResponse, sctsFromTLSExtension,
                                      dcInfo, providerFlags, certVerifierFlags);
@@ -1167,31 +1024,36 @@ SECStatus AuthCertificateHookWithInfo(
 NS_IMPL_ISUPPORTS_INHERITED0(SSLServerCertVerificationResult, Runnable)
 
 SSLServerCertVerificationResult::SSLServerCertVerificationResult(
-    TransportSecurityInfo* infoObject)
+    CommonSocketControl* socketControl)
     : Runnable("psm::SSLServerCertVerificationResult"),
-      mInfoObject(infoObject),
+      mSocketControl(socketControl),
       mCertificateTransparencyStatus(0),
       mEVStatus(EVStatus::NotEV),
       mSucceeded(false),
       mFinalError(0),
-      mCollectedErrors(0),
+      mOverridableErrorCategory(
+          nsITransportSecurityInfo::OverridableErrorCategory::ERROR_UNSET),
       mProviderFlags(0) {}
 
 void SSLServerCertVerificationResult::Dispatch(
     nsTArray<nsTArray<uint8_t>>&& aBuiltChain,
     nsTArray<nsTArray<uint8_t>>&& aPeerCertChain,
     uint16_t aCertificateTransparencyStatus, EVStatus aEVStatus,
-    bool aSucceeded, PRErrorCode aFinalError, uint32_t aCollectedErrors,
-    bool aIsBuiltCertChainRootBuiltInRoot, uint32_t aProviderFlags) {
+    bool aSucceeded, PRErrorCode aFinalError,
+    nsITransportSecurityInfo::OverridableErrorCategory
+        aOverridableErrorCategory,
+    bool aIsBuiltCertChainRootBuiltInRoot, uint32_t aProviderFlags,
+    bool aMadeOCSPRequests) {
   mBuiltChain = std::move(aBuiltChain);
   mPeerCertChain = std::move(aPeerCertChain);
   mCertificateTransparencyStatus = aCertificateTransparencyStatus;
   mEVStatus = aEVStatus;
   mSucceeded = aSucceeded;
   mFinalError = aFinalError;
-  mCollectedErrors = aCollectedErrors;
+  mOverridableErrorCategory = aOverridableErrorCategory;
   mIsBuiltCertChainRootBuiltInRoot = aIsBuiltCertChainRootBuiltInRoot;
   mProviderFlags = aProviderFlags;
+  mMadeOCSPRequests = aMadeOCSPRequests;
 
   if (mSucceeded && mBuiltChain.IsEmpty()) {
     MOZ_ASSERT_UNREACHABLE(
@@ -1234,35 +1096,33 @@ SSLServerCertVerificationResult::Run() {
     SaveIntermediateCerts(mBuiltChain);
   }
 
-  if (mSucceeded) {
-    // Certificate verification succeeded. Delete any potential record of
-    // certificate error bits.
-    RememberCertErrorsTable::GetInstance().RememberCertHasError(mInfoObject,
-                                                                SECSuccess);
+  mSocketControl->SetMadeOCSPRequests(mMadeOCSPRequests);
 
+  if (mSucceeded) {
     MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
             ("SSLServerCertVerificationResult::Run setting NEW cert"));
     nsTArray<uint8_t> certBytes(mBuiltChain.ElementAt(0).Clone());
     nsCOMPtr<nsIX509Cert> cert(new nsNSSCertificate(std::move(certBytes)));
-    mInfoObject->SetServerCert(cert, mEVStatus);
-    mInfoObject->SetSucceededCertChain(std::move(mBuiltChain));
+    mSocketControl->SetServerCert(cert, mEVStatus);
+    mSocketControl->SetSucceededCertChain(std::move(mBuiltChain));
 
-    mInfoObject->SetIsBuiltCertChainRootBuiltInRoot(
+    mSocketControl->SetIsBuiltCertChainRootBuiltInRoot(
         mIsBuiltCertChainRootBuiltInRoot);
-    mInfoObject->SetCertificateTransparencyStatus(
+    mSocketControl->SetCertificateTransparencyStatus(
         mCertificateTransparencyStatus);
   } else {
     nsTArray<uint8_t> certBytes(mPeerCertChain.ElementAt(0).Clone());
     nsCOMPtr<nsIX509Cert> cert(new nsNSSCertificate(std::move(certBytes)));
     // Certificate validation failed; store the peer certificate chain on
-    // infoObject so it can be used for error reporting.
-    mInfoObject->SetFailedCertChain(std::move(mPeerCertChain));
-    if (mCollectedErrors != 0) {
-      mInfoObject->SetStatusErrorBits(cert, mCollectedErrors);
+    // mSocketControl so it can be used for error reporting.
+    mSocketControl->SetFailedCertChain(std::move(mPeerCertChain));
+    if (mOverridableErrorCategory !=
+        nsITransportSecurityInfo::OverridableErrorCategory::ERROR_UNSET) {
+      mSocketControl->SetStatusErrorBits(cert, mOverridableErrorCategory);
     }
   }
 
-  mInfoObject->SetCertVerificationResult(mFinalError);
+  mSocketControl->SetCertVerificationResult(mFinalError);
   return NS_OK;
 }
 
