@@ -36,11 +36,11 @@
 "use strict";
 do_get_profile(); // must be called before getting nsIX509CertDB
 
-const { RemoteSecuritySettings } = ChromeUtils.import(
-  "resource://gre/modules/psm/RemoteSecuritySettings.jsm"
+const { RemoteSecuritySettings } = ChromeUtils.importESModule(
+  "resource://gre/modules/psm/RemoteSecuritySettings.sys.mjs"
 );
-const { TestUtils } = ChromeUtils.import(
-  "resource://testing-common/TestUtils.jsm"
+const { TestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/TestUtils.sys.mjs"
 );
 
 const { CRLiteFiltersClient } = RemoteSecuritySettings.init();
@@ -98,7 +98,7 @@ function getFilenameForFilter(filter) {
  * fake records.
  *
  * @param {*} filters List of filters for which we will create records.
- * @param {Boolean} clear Whether or not to clear the local DB first. Defaults
+ * @param {boolean} clear Whether or not to clear the local DB first. Defaults
  *                        to true.
  */
 async function syncAndDownload(filters, clear = true) {
@@ -271,6 +271,29 @@ add_task(async function test_crlite_filters_basic() {
   );
 });
 
+add_task(async function test_crlite_filters_not_cached() {
+  Services.prefs.setBoolPref(CRLITE_FILTERS_ENABLED_PREF, true);
+  let filters = [
+    { timestamp: "2019-01-01T00:00:00Z", type: "full", id: "0000" },
+  ];
+  let result = await syncAndDownload(filters);
+  equal(
+    result,
+    "finished;2019-01-01T00:00:00Z-full",
+    "CRLite filter download should have run"
+  );
+
+  let records = await CRLiteFiltersClient.client.db.list();
+
+  // `syncAndDownload` should not cache the attachment, so this download should
+  // get the attachment from the source.
+  let attachment = await CRLiteFiltersClient.client.attachments.download(
+    records[0]
+  );
+  equal(attachment._source, "remote_match");
+  await CRLiteFiltersClient.client.attachments.deleteDownloaded(records[0]);
+});
+
 add_task(async function test_crlite_filters_full_and_incremental() {
   Services.prefs.setBoolPref(CRLITE_FILTERS_ENABLED_PREF, true);
 
@@ -400,10 +423,8 @@ add_task(async function test_crlite_confirm_revocations_mode() {
   let certdb = Cc["@mozilla.org/security/x509certdb;1"].getService(
     Ci.nsIX509CertDB
   );
-  let issuerCert = constructCertFromFile("test_crlite_filters/issuer.pem");
-  let noSCTCertIssuer = constructCertFromFile(
-    "test_crlite_filters/no-sct-issuer.pem"
-  );
+  addCertFromFile(certdb, "test_crlite_filters/issuer.pem", ",,");
+  addCertFromFile(certdb, "test_crlite_filters/no-sct-issuer.pem", ",,");
 
   let result = await syncAndDownload([
     {
@@ -446,8 +467,38 @@ add_task(async function test_crlite_confirm_revocations_mode() {
   );
 
   // OCSP should be consulted for this certificate, but OCSP is disabled by
-  // Ci.nsIX509CertDB.FLAG_LOCAL_ONLY so this will return Success.
+  // Ci.nsIX509CertDB.FLAG_LOCAL_ONLY so this will be treated as a soft-failure
+  // and the CRLite result will be used.
   let revokedCert = constructCertFromFile("test_crlite_filters/revoked.pem");
+  await checkCertErrorGenericAtTime(
+    certdb,
+    revokedCert,
+    SEC_ERROR_REVOKED_CERTIFICATE,
+    certificateUsageSSLServer,
+    new Date("2020-10-20T00:00:00Z").getTime() / 1000,
+    undefined,
+    "us-datarecovery.com",
+    Ci.nsIX509CertDB.FLAG_LOCAL_ONLY
+  );
+
+  // Reload the filter w/o coverage and enrollment metadata.
+  result = await syncAndDownload([
+    {
+      timestamp: "2020-10-17T00:00:00Z",
+      type: "full",
+      id: "0000",
+      coverage: [],
+      enrolledIssuers: [],
+    },
+  ]);
+  equal(
+    result,
+    "finished;2020-10-17T00:00:00Z-full",
+    "CRLite filter download should have run"
+  );
+
+  // OCSP will be consulted for the revoked certificate, but a soft-failure
+  // should now result in a Success return.
   await checkCertErrorGenericAtTime(
     certdb,
     revokedCert,
@@ -458,24 +509,6 @@ add_task(async function test_crlite_confirm_revocations_mode() {
     "us-datarecovery.com",
     Ci.nsIX509CertDB.FLAG_LOCAL_ONLY
   );
-
-  // Switch back to enforcement to confirm that it was the security.pki.crlite_mode
-  // that caused us to return Success for revokedCert.
-  Services.prefs.setIntPref(
-    "security.pki.crlite_mode",
-    CRLiteModeEnforcePrefValue
-  );
-
-  await checkCertErrorGenericAtTime(
-    certdb,
-    revokedCert,
-    SEC_ERROR_REVOKED_CERTIFICATE,
-    certificateUsageSSLServer,
-    new Date("2020-10-20T00:00:00Z").getTime() / 1000,
-    undefined,
-    "us-datarecovery.com",
-    0
-  );
 });
 
 add_task(async function test_crlite_filters_and_check_revocation() {
@@ -489,10 +522,8 @@ add_task(async function test_crlite_filters_and_check_revocation() {
   let certdb = Cc["@mozilla.org/security/x509certdb;1"].getService(
     Ci.nsIX509CertDB
   );
-  let issuerCert = constructCertFromFile("test_crlite_filters/issuer.pem");
-  let noSCTCertIssuer = constructCertFromFile(
-    "test_crlite_filters/no-sct-issuer.pem"
-  );
+  addCertFromFile(certdb, "test_crlite_filters/issuer.pem", ",,");
+  addCertFromFile(certdb, "test_crlite_filters/no-sct-issuer.pem", ",,");
 
   let result = await syncAndDownload([
     {
